@@ -1,101 +1,173 @@
 # gridData --- python modules to read and write gridded data
-# Copyright (c) 2009-2010 Oliver Beckstein <orbeckst@gmail.com>
-# Released under the GNU Lesser Public License, version 3 or later.
+# Copyright (c) 2009-2014 Oliver Beckstein <orbeckst@gmail.com>
+# Released under the GNU Lesser General Public License, version 3 or later.
 
-"""
-:mod:`OpenDX` --- routines to read and write simple OpenDX files
-================================================================
+r"""
+:mod:`~gridData.OpenDX` --- routines to read and write simple OpenDX files
+==========================================================================
 
-The OpenDX format for multi-dimensional grid data. This
-module only implements a primitive subset, sufficient to represent
-n-dimensional regular grids. OpenDX is a free visualization software,
-see http://www.opendx.org.
+The OpenDX format for multi-dimensional grid data. OpenDX is a free
+visualization software, see http://www.opendx.org.
 
-See http://opendx.sdsc.edu/docs/html/pages/usrgu068.htm#HDREDF for the
-file format specifications.
+.. Note:: This module only implements a primitive subset, sufficient
+          to represent n-dimensional regular grids.
+
+The OpenDX scalar file format is specified in Appendix `B.2 Data
+Explorer Native Files`_ [#OpenDXformat]_.
 
 If you want to build a dx object from your data you can either use the
-convenient Grid class from the top level module (gridData.Grid)
-or see the lower-level methods described below.
+convenient :class:`~gridData.core.Grid` class from the top level
+module (:class:`gridData.Grid`) or see the lower-level methods
+described below.
 
 
-Building a dx object from a numpy array A
------------------------------------------
+.. _opendx-read-write:
 
-If you have a numpy array A that represents a density in cartesian
-space then you can construct a dx object if you provide some
-additional information that fixes the coordinate system in space and
-defines the units along the axes.
+Reading and writing OpenDX files
+--------------------------------
+
+If you have OpenDX files from other software and you just want to
+**read** it into a Python array then you do not really need to use the
+interface in :mod:`gridData.OpenDX`: just use
+:class:`~gridData.core.Grid` and load the file::
+
+  from gridData import Grid
+  g = Grid("data.dx")
+
+This should work for files produced by common visualization programs
+(VMD_, PyMOL_, Chimera_). The documentation for :mod:`gridData` tells
+you more about what to do with the :class:`~gridData.core.Grid`
+object.
+
+If you want to **write** an OpenDX file then you just use the
+:meth:`gridData.core.Grid.export` method with `file_format="dx"` (or
+just use a filename with extension ".dx")::
+
+  g.export("data.dx")
+
+However, some visualization programs do not implement full OpenDX
+specifications and only read very specific, "OpenDX-like"
+files. :mod:`gridData.OpenDX` tries to be compatible with these
+formats. However, sometimes additional help is needed to write an
+OpenDX file that can be read by a specific software, as described
+below:
+
+Known issues for writing OpenDX files
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+* PyMOL_ requires OpenDX files with the type specification "double" in
+  the `class array` section (see issue `#35`_). By default (since
+  release 0.4.0), the type is set to the one that most closely
+  approximates the dtype of the numpy array :attr:`Grid.grid`, which
+  holds all data. This is often :class:`numpy.float64`, which will
+  create an OpenDX type "double", which PyMOL will read.
+
+  However, if you want to *force* a specific OpenDX type (such as
+  "float" or "double", see :attr:`gridData.OpenDX.array.dx_types` for
+  available values) then you can use the ``type`` keyword argument::
+
+    g.export("for_pymol.dx", type="double")
+
+  If you always want to be able to read OpenDX files with PyMOL, it is
+  suggested to always export with ``type="double"``.
+
+  .. versionadded:: 0.4.0
+
+
+
+.. _VMD: http://www.ks.uiuc.edu/Research/vmd/
+.. _PyMOL: http://www.pymol.org/
+.. _Chimera: https://www.cgl.ucsf.edu/chimera/
+.. _`#35`: https://github.com/MDAnalysis/GridDataFormats/issues/35
+
+
+
+
+Building a dx object from a numpy array ``A``
+---------------------------------------------
+
+If you have a numpy array ``A`` that represents a density in cartesian
+space then you can construct a dx object (named a *field* in OpenDX
+parlance) if you provide some additional information that fixes the
+coordinate system in space and defines the units along the axes.
 
 The following data are required:
 
-   grid              numpy nD array (typically a nD histogram)
-   grid.shape        the shape of the array
-   origin            the cartesian coordinates of the center of the 
-                     (0,0,..,0) grid cell
-   delta             n x n array with the length of a grid cell along 
-                     each axis; for regular rectangular grids the off-diagonal 
-                     elements are 0 and the diagonal ones correspond to the 
-                     'bin width' of the histogram, eg delta[0,0] = 1.0 (Angstrom)
+grid
+    numpy nD array (typically a nD histogram)
+grid.shape
+    the shape of the array
+origin
+    the cartesian coordinates of the center of the (0,0,..,0) grid cell
+delta
+    :math:`n \times n` array with the length of a grid cell along
+    each axis; for regular rectangular grids the off-diagonal
+    elements are 0 and the diagonal ones correspond to the
+    'bin width' of the histogram, eg ``delta[0,0] = 1.0`` (Angstrom)
 
->>> dx = OpenDX.field('density')
->>> dx.add('positions',OpenDX.gridpositions(1,grid.shape,origin,delta))
->>> dx.add('connections',OpenDX.gridconnections(2,grid.shape))
->>> dx.add('data',OpenDX.array(3,grid))
+The DX data type ("type" in the DX file) is determined from the
+:class:`numpy.dtype` of the :class:`numpy.ndarray` that is provided as
+the *grid* (or with the *type* keyword argument to
+:class:`gridData.OpenDX.array`).
 
-or
+For example, to build a :class:`field`::
 
->>> dx = OpenDX.field('density',components=dict(
-            positions=OpenDX.gridpositions(1,grid.shape,d.origin,d.delta), 
-            connections=OpenDX.gridconnections(2,grid.shape), 
-            data=OpenDX.array(3,grid)) )
+  dx = OpenDX.field('density')
+  dx.add('positions', OpenDX.gridpositions(1, grid.shape, origin, delta))
+  dx.add('connections', OpenDX.gridconnections(2, grid.shape))
+  dx.add('data', OpenDX.array(3, grid))
+
+or all with the constructor::
+
+  dx = OpenDX.field('density', components=dict(
+            positions=OpenDX.gridpositions(1,grid.shape, d.origin, d.delta),
+            connections=OpenDX.gridconnections(2, grid.shape),
+            data=OpenDX.array(3, grid)))
 
 
 Building a dx object from a dx file
 -----------------------------------
 
-One can also read data from an existing dx file:
+One can also read data from an existing dx file::
 
->>> dx = OpenDX.field(0)
->>> dx.read('file.dx')
+ dx = OpenDX.field(0)
+ dx.read('file.dx')
 
-The dx object has a method histogramdd() that produces output identical
-to the numpy.histogramdd() function. In this way, one can store nD
-histograms in a portable and universal manner:
+Only simple arrays are read and initially stored as a 1-d
+:class:`numpy.ndarray` in the `dx.components['data'].array` with the
+:class:`numpy.dtype` determined by the DX type in the file.
 
->>> histogram,edges = dx.histogramdd()
+The dx :class:`field` object has a method
+:meth:`~OpenDX.field.histogramdd` that produces output identical to
+the :func:`numpy.histogramdd` function by taking the stored dimension
+and deltas into account. In this way, one can store nD histograms in a
+portable and universal manner::
+
+  histogram, edges = dx.histogramdd()
+
+.. rubric:; Footnotes
+
+.. [#OpenDXformat] The original link to the OpenDX file format specs
+   http://opendx.sdsc.edu/docs/html/pages/usrgu068.htm#HDREDF is dead so I am linking
+   to an archived copy at the Internet Archive , `B.2 Data Explorer Native Files`_.
+
+.. _`B.2 Data Explorer Native Files`:
+   https://web.archive.org/web/20080808140524/http://opendx.sdsc.edu/docs/html/pages/usrgu068.htm
+.. http://opendx.sdsc.edu/docs/html/pages/usrgu068.htm#HDREDF
+
+Classes and functions
+---------------------
 
 """
 from __future__ import with_statement
 
 import numpy
 import re
+from six import next
+from six.moves import range
 
-try:
-    sorted([])
-except NameError:
-    def sorted(iterable, cmp=None, key=None, reverse=False):
-        """sorted(iterable, cmp=None, key=None, reverse=False) --> new sorted list
-        Naive pre python 2.4 compatibility fudge.
+import warnings
 
-        With key, cmp must make use of triplets (key,int,value).
-        It's a fudge after all. 
-        """
-        L = list(iterable)
-        args = ()
-        if cmp is not None:
-            args = (cmp,)
-        if key is None:
-            L.sort(*args)
-        else:
-            # decorate-sort-undecorate
-            deco = [(key(x),i,x) for i,x in enumerate(L)]
-            deco.sort(*args)
-            L[:] = [y[2] for y in deco] 
-        if reverse:
-            L.reverse()
-        return L
-    
 class DXclass(object):
     """'class' object as defined by OpenDX"""
     def __init__(self,classid):
@@ -104,6 +176,7 @@ class DXclass(object):
         self.name = None   # name of the DXclass
         self.component = None   # component type
         self.D = None      # dimensions
+
     def write(self,file,optstring="",quote=False):
         """write the 'object' line; additional args are packed in string"""
         classid = str(self.id)
@@ -113,7 +186,7 @@ class DXclass(object):
         # spaces. (Chimera 1.4.1, PyMOL 1.3)
         file.write('object '+classid+' class '+str(self.name)+' '+\
                    optstring+'\n')
-        
+
     def read(self,file):
         raise NotImplementedError('Reading is currently not supported.')
 
@@ -124,7 +197,7 @@ class DXclass(object):
 
     def __repr__(self):
         return '<OpenDX.'+str(self.name)+' object, id='+str(self.id)+'>'
-    
+
 
 class gridpositions(DXclass):
     """OpenDX gridpositions class.
@@ -141,11 +214,19 @@ class gridpositions(DXclass):
         self.component = 'positions'
         self.shape = numpy.asarray(shape)      # D dimensional shape
         self.origin = numpy.asarray(origin)    # D vector
-        self.delta = numpy.asarray(delta)      # DxD array of grid spacings
         self.rank = len(self.shape)            # D === rank
-        if self.delta.shape != (self.rank,self.rank):
-            # check OpenDX specs for irreg spacing
-            raise NotImplementedError('Only regularly spaced grids allowed.')
+
+        self.delta = numpy.asarray(delta)      # DxD array of grid spacings
+        # gridDataFormats  actually provides a simple 1D array with the deltas because only
+        # regular grids are used but the following is a reminder that OpenDX should be able
+        # to handle more complicated volume elements
+        if len(self.delta.shape) == 1:
+            self.delta = numpy.diag(delta)
+        if self.delta.shape != (self.rank, self.rank):
+            # check OpenDX specs for irreg spacing if we want to implement
+            # anything more complicated
+            raise NotImplementedError('Only regularly spaced grids allowed, '
+                                      'not delta={}'.format(self.delta))
     def write(self,file):
         DXclass.write(self,file,
                       ('counts '+self.ndformat(' %d')) % tuple(self.shape))
@@ -158,7 +239,7 @@ class gridpositions(DXclass):
         Only works for regular, orthonormal grids.
         """
         return [self.delta[d,d] * numpy.arange(self.shape[d]+1) + self.origin[d]\
-                - 0.5*self.delta[d,d]     for d in xrange(self.rank)]
+                - 0.5*self.delta[d,d]     for d in range(self.rank)]
 
 
 class gridconnections(DXclass):
@@ -175,27 +256,129 @@ class gridconnections(DXclass):
                       ('counts '+self.ndformat(' %d')) % tuple(self.shape))
 
 class array(DXclass):
-    """OpenDX array class"""
-    def __init__(self,classid,array=None,**kwargs):
+    """OpenDX array class.
+
+    See `Array Objects`_ for details.
+
+    .. _Array Objects:
+       https://web.archive.org/web/20080808140524/http://opendx.sdsc.edu/docs/html/pages/usrgu068.htm#Header_440
+    """
+    #: conversion from :attr:`numpy.dtype.name` to closest OpenDX array type
+    #: (round-tripping is not guaranteed to produce identical types); not all
+    #: types are supported (e.g., strings are missing)
+    np_types = {
+        "uint8": "byte",         # DX "unsigned byte" equivalent
+        "int8": "signed byte",
+        "uint16": "unsigned short",
+        "int16": "short",         # DX "signed short" equivalent
+        "uint32": "unsigned int",
+        "int32": "int",           # DX "signed int"   equivalent
+        "uint64": "unsigned int", # not explicit in DX, for compatibility
+        "int64": "int",           # not explicit in DX, for compatibility
+        # "hyper",                # ?
+        "float32": "float",       # default
+        "float64": "double",
+        "float16": "float",       # float16 not available in DX, use float
+        # numpy "float128 not available, raise error
+        # "string" not automatically supported
+    }
+    #: conversion from OpenDX type to closest :class:`numpy.dtype`
+    #: (round-tripping is not guaranteed to produce identical types); not all
+    #: types are supported (e.g., strings and conversion to int64 are missing)
+    dx_types = {
+        "byte": "uint8",
+        "unsigned byte": "uint8",
+        "signed byte": "int8",
+        "unsigned short": "uint16",
+        "short": "int16",
+        "signed short": "int16",
+        "unsigned int": "uint32",
+        "int": "int32",
+        "signed int": "int32",
+        # "hyper",                # ?
+        "float": "float32",       # default
+        "double": "float64",
+        # "string" not automatically supported
+    }
+
+    def __init__(self, classid, array=None, type=None, **kwargs):
+        """
+        Parameters
+        ----------
+        classid : int
+        array : array_like
+        type : str (optional)
+             Set the DX type in the output file and cast `array` to
+             the closest numpy dtype.  `type` must be one of the
+             allowed types in DX files as defined under `Array
+             Objects`_.  The default ``None`` tries to set the type
+             from the :class:`numpy.dtype` of `array`.
+
+             .. versionadded:: 0.4.0
+
+        Raises
+        ------
+        ValueError
+             if `array` is not provided; or if `type` is not of the correct
+             DX type
+        """
         if array is None:
             raise ValueError('array keyword argument is required')
         self.id = classid
         self.name = 'array'
         self.component = 'data'
-        self.array = numpy.asarray(array)
-    def write(self,file):
+        # detect type https://github.com/MDAnalysis/GridDataFormats/issues/35
+        if type is None:
+            self.array = numpy.asarray(array)
+            try:
+                self.type = self.np_types[self.array.dtype.name]
+            except KeyError:
+                warnings.warn(("array dtype.name = {0} can not be automatically "
+                               "converted to a DX array type. Use the 'type' keyword "
+                               "to manually specify the correct type.").format(
+                                   self.array.dtype.name))
+                self.type = self.array.dtype.name  # will raise ValueError on writing
+        else:
+            try:
+                self.array = numpy.asarray(array, dtype=self.dx_types[type])
+            except KeyError:
+                raise ValueError(("DX type {0} cannot be converted to an "
+                                  "appropriate numpy dtype. Available "
+                                  "types are: {1}".format(type,
+                                                          list(self.dx_types.values()))))
+            self.type = type
+
+    def write(self, file):
+        """Write the *class array* section.
+
+        Parameters
+        ----------
+        file : file
+
+        Raises
+        ------
+        ValueError
+             If the `dxtype` is not a valid type, :exc:`ValueError` is
+             raised.
+
+        """
+        if self.type not in self.dx_types:
+            raise ValueError(("DX type {} is not supported in the DX format. \n"
+                              "Supported valus are: {}\n"
+                              "Use the type=<type> keyword argument.").format(
+                                  self.type, list(self.dx_types.keys())))
         DXclass.write(self,file,
-                      'type float rank 0 items %d data follows' % \
-                      self.array.size)
+                      'type "{0}" rank 0 items {1} data follows'.format(
+                          self.type, self.array.size))
         # grid data, serialized as a C array (z fastest varying)
         # (flat iterator is equivalent to: for x: for y: for z: grid[x,y,z])
-        # VMD's DX reader requires exactly 3 values per line 
+        # VMD's DX reader requires exactly 3 values per line
         values_per_line = 3
-        anext = self.array.flat.next
+        values = self.array.flat
         while 1:
             try:
-                for i in xrange(values_per_line):
-                    file.write(str(anext())+"\t")  # I hope this is written even if the try fails..
+                for i in range(values_per_line):
+                    file.write(str(next(values)) + "\t")
                 file.write('\n')
             except StopIteration:
                 file.write('\n')
@@ -205,39 +388,36 @@ class array(DXclass):
 class field(DXclass):
     """OpenDX container class
 
-        Methods:
+    The *field* is the top-level object and represents the whole
+    OpenDX file. It contains a number of other objects.
 
-        write(file)     write OpenDX file to file descriptor; only simple
-                        regular arrays are supported. File should be readable
-                        by VMD.
-        append(obj)     add a DXclass object
+    Instantiate a DX object from this class and add subclasses with
+    :meth:`add`.
 
-    Derive a DX instance from this class and add subclasses with append.
     """
     # perhaps this should not derive from DXclass as those are
     # objects in field but a field cannot contain itself
     def __init__(self,classid='0',components=None,comments=None):
         """OpenDX object, which is build from a list of components.
 
-        dx = OpenDX.field('density',[gridpoints,gridconnections,array])
+        Parameters
+        ----------
 
-        Arguments:
+        id : str
+               arbitrary string
+        components : dict
+               dictionary of DXclass instances (no sanity check on the
+               individual ids!) which correspond to
+
+               * positions
+               * connections
+               * data
+
+        comments : list
+               list of strings; each string becomes a comment line
+               prefixed with '#'. Avoid newlines.
 
-        id              arbitrary string
-        components      dictionary of DXclass instances (no sanity check on the
-                        individual ids!) which correspond to
-                          positions
-                          connections
-                          data        
-        comments        list of strings; each string becomes a comment line
-                        prefixed with '#'. Avoid newlines.
 
-        Methods:
-        read()          constrcut the field from a dx file
-        write()         write a dx file
-        add()           add a component to the field
-        add_comments()  add comments
-        
         A field must have at least the components 'positions',
         'connections', and 'data'. Those components are associated
         with objects belonging to the field. When writing a dx file
@@ -249,13 +429,21 @@ class field(DXclass):
         objects are written and then the field object describes its
         components. Objects are referenced by their unique id.)
 
-        NOTE: uniqueness of the id is not checked.
+        .. Note:: uniqueness of the *id* is not checked.
+
+
+        Example
+        -------
+        Create a new dx object::
+
+           dx = OpenDX.field('density',[gridpoints,gridconnections,array])
+
         """
         if components is None:
             components = dict(positions=None,connections=None,data=None)
         if comments is None:
             comments = ['OpenDX written by gridData.OpenDX',
-                        'from http://github.com/orbeckst/GridDataFormats']
+                        'from https://github.com/MDAnalysis/GridDataFormats']
         elif type(comments) is not list:
             comments = [str(comments)]
         self.id = classid       # can be an arbitrary string
@@ -264,14 +452,14 @@ class field(DXclass):
         self.components = components
         self.comments= comments
 
-    def write(self,filename):
+    def write(self, filename):
         """Write the complete dx object to the file.
 
-        write(filename)
-
         This is the simple OpenDX format which includes the data into
         the header via the 'object array ... data follows' statement.
 
+        Only simple regular arrays are supported.
+
         The format should be compatible with VMD's dx reader plugin.
         """
         # comments (VMD chokes on lines of len > 80, so truncate)
@@ -291,7 +479,7 @@ class field(DXclass):
     def read(self,file):
         """Read DX field from file.
 
-        dx = OpenDX.field.read(dxfile)
+            dx = OpenDX.field.read(dxfile)
 
         The classid is discarded and replaced with the one from the file.
         """
@@ -300,16 +488,19 @@ class field(DXclass):
         p.parse(DXfield)
 
     def add(self,component,DXobj):
+        """add a component to the field"""
         self[component] = DXobj
 
     def add_comment(self,comment):
+        """add comments"""
         self.comments.append(comment)
 
     def sorted_components(self):
         """iterator that returns (component,object) in id order"""
-        for component,object in \
-                sorted(self.components.items(),key=lambda (c,o):o.id):
-            yield component,object
+        for component, object in \
+                sorted(self.components.items(),
+                       key=lambda comp_obj: comp_obj[1].id):
+            yield component, object
 
     def histogramdd(self):
         """Return array data as (edges,grid), i.e. a numpy nD histogram."""
@@ -405,7 +596,7 @@ class DXParser(object):
     """Brain-dead baroque implementation to read a simple (VMD) dx file.
 
     Requires a OpenDX.field instance.
-    
+
     1) scan for 'object' lines:
        'object' id 'class' class  [data]
        [data ...]
@@ -452,12 +643,15 @@ class DXParser(object):
                         'array':self.__array, 'field':self.__field,
                         }
 
-        
+
     def parse(self,DXfield):
         """Parse the dx file and construct a DX field object with component classes.
 
-        DXfield_object = OpenDX.field(*args)
-        parse(DXfield_object)        
+        A :class:`field` instance *DXfield* must be provided to be
+        filled by the parser::
+
+           DXfield_object = OpenDX.field(*args)
+           parse(DXfield_object)
 
         A tokenizer turns the dx file into a stream of tokens. A
         hierarchy of parsers examines the stream. The level-0 parser
@@ -471,7 +665,7 @@ class DXParser(object):
           not implemented yet.
         * Unknown tokens raise an exception.
         """
-    
+
         self.DXfield = DXfield              # OpenDX.field (used by comment parser)
         self.currentobject = None           # containers for data
         self.objects = []                   # |
@@ -495,12 +689,12 @@ class DXParser(object):
 
         # free space
         del self.currentobject, self.objects
-        
+
 
 
     def __general(self):
         """Level-0 parser and main loop.
-        
+
         Look for a token that matches a level-1 parser and hand over control."""
         while 1:                            # main loop
             try:
@@ -510,7 +704,7 @@ class DXParser(object):
                 # (kludge in here as the last level-2 parser usually does not return
                 # via the object parser)
                 if self.currentobject and self.currentobject not in self.objects:
-                    self.objects.append(self.currentobject)                        
+                    self.objects.append(self.currentobject)
                 return                      # stop parsing and finish
             # decision branches for all level-1 parsers:
             # (the only way to get out of the lower level parsers!)
@@ -525,7 +719,7 @@ class DXParser(object):
                 # later we never formally switch back to __general
                 # (would create inifinite loop)
                 raise DXParseError('Unknown level-1 construct at '+str(tok))
-                
+
             self.apply_parser()     # hand over to new parser
                                     # (possibly been set further down the hierarchy!)
 
@@ -559,7 +753,7 @@ class DXParser(object):
         # setup new DXInitObject
         classtype = self.__consume().text
         self.currentobject = DXInitObject(classtype=classtype,classid=classid)
-        
+
         self.use_parser(classtype)
 
     # Level-2 parser (object parsers)
@@ -572,12 +766,12 @@ class DXParser(object):
         delta 1 0 0
         delta 0 1 0
         delta 0 0 1
-        """        
+        """
         try:
             tok = self.__consume()
         except DXParserNoTokens:
             return
-            
+
         if tok.equals('counts'):
             shape = []
             try:
@@ -592,7 +786,7 @@ class DXParser(object):
         elif tok.equals('origin'):
             origin = []
             try:
-                while (self.__peek().iscode('INTEGER') or 
+                while (self.__peek().iscode('INTEGER') or
                        self.__peek().iscode('REAL')):
                     tok = self.__consume()
                     origin.append(tok.value())
@@ -606,7 +800,7 @@ class DXParser(object):
             try:
                 while (self.__peek().iscode('INTEGER') or
                        self.__peek().iscode('REAL')):
-                    tok = self.__consume()                        
+                    tok = self.__consume()
                     d.append(tok.value())
             except DXParserNoTokens:
                 pass
@@ -618,7 +812,7 @@ class DXParser(object):
                 self.currentobject['delta'] = [d]
         else:
             raise DXParseError('gridpositions: '+str(tok)+' not recognized.')
-            
+
 
     def __gridconnections(self):
         """Level-2 parser for gridconnections.
@@ -631,7 +825,7 @@ class DXParser(object):
             tok = self.__consume()
         except DXParserNoTokens:
             return
-            
+
         if tok.equals('counts'):
             shape = []
             try:
@@ -647,7 +841,7 @@ class DXParser(object):
             raise DXParseError('gridconnections: '+str(tok)+' not recognized.')
 
 
-    def __array(self):        
+    def __array(self):
         """Level-2 parser for arrays.
 
         pattern:
@@ -662,7 +856,7 @@ class DXParser(object):
             tok = self.__consume()
         except DXParserNoTokens:
             return
-    
+
         if tok.equals('type'):
             tok = self.__consume()
             if not tok.iscode('STRING'):
@@ -692,7 +886,7 @@ class DXParser(object):
             if not self.currentobject['size']:
                 raise DXParseError("array: missing number of items")
             self.currentobject['array'] = [self.__consume().value('REAL') \
-                                           for i in xrange(self.currentobject['size'])]
+                                           for i in range(self.currentobject['size'])]
         elif tok.equals('attribute'):
             # not used at the moment
             attribute = self.__consume().value()
@@ -706,7 +900,7 @@ class DXParser(object):
         """Level-2 parser for a DX field object.
 
         pattern:
-        object "site map 1" class field  
+        object "site map 1" class field
         component "positions" value 1
         component "connections" value 2
         component "data" value 3
@@ -790,9 +984,8 @@ class DXParser(object):
           pass
         """
         self.__refill_tokenbuffer()
-        #print "DEBUG consume: "+str(self.__parser)+' '+str(self.__peek())        
+        #print "DEBUG consume: "+str(self.__parser)+' '+str(self.__peek())
         try:
             return self.tokens.pop(0)  # singlet
         except IndexError:
             raise DXParserNoTokens
-        
